==> patch-4.1.6-ONex-OMS-wms-source-PHP5.3.tar.gz: ShopexErp/app/erpapi/lib/shop/response/exchange.php <==
<?php
/**
 * @desc    换货对接
 * @author:sunjing@shopex.cn
 * @since:
 */
class erpapi_shop_response_exchange extends erpapi_shop_response_abstract {
/*买家已经申请退款，等待卖家同意                 WAIT_SELLER_AGREE
 卖家已经同意换货，等待买家退货                WAIT_BUYER_SEND_GOODS
 买家已经退货，等待卖家确认收货                WAIT_SELLER_CONFIRM_GOODS
 卖家拒绝确认收货                               SELLER_REFUSED_CONFIRM_GOODS
 换货关闭                                       EXCHANGE_CLOSE
 换货成功                                       EXCHANGE_SUCCESS
 换货结束                                       EXCHANGE_FINISH
 卖家确认收货,等待卖家发货                      WAIT_SELLER_SEND_GOODS
 换货关闭,转退货退款                             EXCHANGE_CLOSE_TO_SALES_RETURN*/

    static public  $return_status = array(
        'WAIT_SELLER_AGREE'         =>'1',//换货待处理
        'WAIT_BUYER_SEND_GOODS'     =>'3',//待买家退货
        'WAIT_SELLER_CONFIRM_GOODS'  =>'6',//  买家已退货，待收货
        'EXCHANGE_CLOSE'            =>'5',// 换货关闭
        'EXCHANGE_SUCCESS'          =>'4',// 换货成功
        'EXCHANGE_FINISH'           =>  '4',
        'WAIT_SELLER_SEND_GOODS'    =>'4',// 待发出换货商品  转换成  待收货状态？没这个状态的
        'SELLER_REFUSED_CONFIRM_GOODS'=>'9',// 卖家拒绝确认收货
        'EXCHANGE_CLOSE_TO_SALES_RETURN'=>'5',//  请退款
    );



    protected function _formatAddParams($params) {
        self::trim($params);
        return $params;
    }


    protected function _returnProductAdditional($sdf) {
        return array();
    }

    public function add($params){

        $original_bn = $params['dispute_id'];
        $change_conf = app::get('ome')->getConf('shop.tmallchange.config.'.$this->__channelObj->channel['shop_id']);
        if (!$change_conf){
            $this->__apilog['result']['msg'] = '未开启接收换货业务';
            return false;

        }

        $sdf = $this->_formatAddParams($params);
        if ($sdf){
            $original_bn = $sdf['order']['order_bn'];
        }
        $this->__apilog['title'] = '店铺(' . $this->__channelObj->channel['name'] . ')换货业务处理[换货单号：' . $params['dispute_id']. ']';
        $this->__apilog['original_bn'] = $original_bn;
        $this->__apilog['result']['data'] = array('tid'=>$original_bn,'aftersale_id'=>$params['dispute_id'],'retry'=>'false');
        if(empty($sdf)) {
            $this->__apilog['result']['msg'] = '不接收售后单';
            return false;
        }
        //是否发货判断
        if (in_array($sdf['order']['ship_status'],array('0'))){
            $this->__apilog['result']['msg'] = '未发货订单不接收售后单';
            return false;
        }
        //判断状态是否处理
        $status = self::$return_status[strtoupper($sdf['status'])];
        if (!$status) {

            $this->__apilog['result']['msg'] = '此状态不处理!';
            return false;
        }
        $shopId = $sdf['shop_id'] = $this->__channelObj->channel['shop_id'];

        if(empty($sdf['return_items'])) {
            $this->__apilog['result']['msg'] = '退货明细不可为空!';
            return false;
        }

        if (is_array($sdf['change_items'])){
            $changeItemList = $this->_formatChangeItemlist($sdf['change_items']);
            if (empty($changeItemList)){
                $this->__apilog['result']['msg'] = '换货明细不可为空!';
                return false;
            }else{
                $sdf['change_items'] = $changeItemList;
            }
        }


        return $this->_returnProductAddSdf($sdf);
    }


    protected function _getOrderDelivery($sdf) {
        #获取订单关联的所有已发货的发货单delivery_id
        $sql = "SELECT dord.delivery_id, d.branch_id FROM sdb_ome_delivery_order AS dord
                  LEFT JOIN sdb_ome_delivery AS d ON(dord.delivery_id=d.delivery_id)
                  WHERE dord.order_id='".$sdf['order']['order_id']."'
                    AND (d.parent_id=0 OR d.is_bind='true')
                    AND d.disabled='false' AND d.status='succ'";

        $result = kernel::database()->select($sql);
        if(count($result) > 1 && !empty($sdf['return_items'])) {
            $arrDelivery = array();
            foreach ($result as $key => $val) {
                $arrDelivery[$val['delivery_id']] = $val;
            }
            $productId = '';
            foreach($sdf['return_items'] as $iVal) {
                if($iVal['product_id']) {
                    $productId = $iVal['product_id'];
                    break;
                }
            }
            $deliItemModel = app::get('ome')->model('delivery_items');
            $itemData = $deliItemModel->getList('delivery_id', array('delivery_id'=>array_keys($arrDelivery), 'product_id'=>$productId), 0, 1);
        }
        return $itemData ? $arrDelivery[$itemData[0]['delivery_id']] : $result[0];
    }



    protected function _returnProductAddSdf($sdf) {

        $sdf['status'] = self::$return_status[strtoupper($sdf['status'])];
        $returnModel = app::get('ome')->model('return_product');
        $tgReturn = $returnModel->getList('*', array('shop_id'=>$sdf['shop_id'],'return_bn'=>$sdf['return_bn']));
        $version_change = false;
        if($tgReturn) {
            $sdf['return_product'] = $tgReturn[0];

            $reshipData = $this->_returnProductReship($sdf['return_product']['return_id'], $sdf['order']['order_id']);
            if($reshipData['reship']) {
                $sdf['reship'] = $reshipData['reship'];
            }
            if($reshipData['other_reship_items']) {
                $sdf['other_reship_items'] = $reshipData['other_reship_items'];
            }

            if ($sdf['modified'] > $sdf['return_product']['outer_lastmodify']){
                $version_change = true;
            }

            if($sdf['modified'] <= $sdf['return_product']['outer_lastmodify']) {
                $this->__apilog['result']['msg'] = '更新时间未变化不更新';
                return false;
            }
        }
        //不存在状态为已完成和已拒绝不处理
        if (!$sdf['return_product'] && in_array($sdf['status'],array('4','5'))){
            $this->__apilog['result']['msg'] = '已完成单据不处理!';
            return false;
        }

        //可退数量判断

        $sdf['version_change'] = $version_change;
        //判断是否已拒绝或完成状态
        if ($sdf['return_product']['status'] == '5' && !$version_change){
            $this->__apilog['result']['msg'] = '已拒绝版本未变化不处理';
            return false;
        }
        if ($sdf['return_product']['status'] == '4'){
            $this->__apilog['result']['msg'] = '已完成售后单不处理';
            return false;
        }
        $delivery = $this->_getOrderDelivery($sdf);
        if ($delivery) {
            $sdf['delivery_id'] = $delivery['delivery_id'];
            $sdf['branch_id'] = $delivery['branch_id'];
        }
        if ($sdf['status'] == '1' || !$sdf['return_product']) {
            // 如果前端传了会员名
            if ($sdf['buyer_nick']) {
                $shopMemberModel = app::get('ome')->model('shop_members');
                $member = $shopMemberModel->getList('member_id', array('shop_member_id' => $sdf['buyer_nick'], 'shop_id' => $sdf['shop_id']), 0, 1);
                $sdf['member_id'] = $member[0]['member_id'];
            } else {
                $sdf['member_id'] = $sdf['order']['member_id'];
            }
            //判断数量
            $return_items = $sdf['return_items'];
            $itemObj = app::get('ome')->model('reship_items');


            foreach($return_items as $item){
                $effective = $itemObj->Get_refund_count ( $sdf['order']['order_id'], $item['bn'] );
                if ($effective<=0)  {
                    $this->__apilog['result']['msg'] = "返回值：货号[{$item['bn']}]申请数超出订单可退数";
                    return false;
                }
            }

            $sdf['table_additional'] = $this->_returnProductAdditional($sdf);
        } else {
            $returnItemModel = app::get('ome')->model('return_product_items');
            $tgReturnItems = $returnItemModel->getList('item_id',array('return_id'=>$sdf['return_product']['return_id']));
            if (!$tgReturnItems) {
                $this->__apilog['result']['msg'] = '缺少明细';
                return false;
            }

            if ($sdf['status']<=3){
                $sdf['table_additional'] = $this->_returnProductAdditional($sdf);
            }
        }
        $sdf['choose_type_flag'] = 1;

        return $sdf;
    }

    protected function _returnProductReship($returnProductId, $orderId) {
        $rs = array();
        $oReship = app::get('ome')->model('reship');
        $field = 'reship_id,reship_bn,return_id,is_check,return_logi_name,return_logi_no,outer_lastmodify,change_status,return_type';
        $reship = $oReship->getList($field, array('return_id' => $returnProductId), 0, 1);
        if($reship) {
            $rs['reship'] = $reship[0];
            $otherFilter = ' AND r.reship_id != ' . $rs['reship']['reship_id'];
        }
        $sql = "SELECT i.bn, i.num FROM sdb_ome_reship as r left join sdb_ome_reship_items as i on r.reship_id=i.reship_id WHERE i.return_type='return' AND r.is_check!='5' AND r.order_id='" . $orderId . "' " . $otherFilter;
        $otherReship = $oReship->db->select($sql);
        if($otherReship) {
            $rs['other_reship_items'] = $otherReship;
        }
        return $rs;
    }


    protected function _formatChangeItemlist($change_items){
        $salesMLib = kernel::single('material_sales_material');
        foreach($change_items as &$items){
            $product_detail = $salesMLib->getSalesMByBn($this->__channelObj->channel['shop_id'],$items['bn']);
            if($product_detail){
                $items['product_id'] = $product_detail['sm_id'];
                $items['name'] = $product_detail['sales_material_name'];
            }else{
                return array();
            }
        }
        return $change_items;
    }


}

==> patch-4.3.0-ONex-OMS-wms-source-PHP5.6.tar.gz: ShopexErp/app/erpapi/lib/shop/response/exchange.php <==
<?php
/**
 * @desc    换货对接
 * @author:sunjing@shopex.cn
 * @since:
 */
class erpapi_shop_response_exchange extends erpapi_shop_response_abstract {
/*买家已经申请退款，等待卖家同意                 WAIT_SELLER_AGREE
 卖家已经同意换货，等待买家退货                WAIT_BUYER_SEND_GOODS
 买家已经退货，等待卖家确认收货                WAIT_SELLER_CONFIRM_GOODS
 卖家拒绝确认收货                               SELLER_REFUSED_CONFIRM_GOODS
 换货关闭                                       EXCHANGE_CLOSE
 换货成功                                       EXCHANGE_SUCCESS
 换货结束                                       EXCHANGE_FINISH
 卖家确认收货,等待卖家发货                      WAIT_SELLER_SEND_GOODS
 换货关闭,转退货退款                             EXCHANGE_CLOSE_TO_SALES_RETURN*/

    static public  $return_status = array(
        'WAIT_SELLER_AGREE'         =>'1',//换货待处理
        'WAIT_BUYER_SEND_GOODS'     =>'3',//待买家退货
        'WAIT_SELLER_CONFIRM_GOODS'  =>'6',//  买家已退货，待收货
        'EXCHANGE_CLOSE'            =>'5',// 换货关闭
        'EXCHANGE_SUCCESS'          =>'4',// 换货成功
        'EXCHANGE_FINISH'           =>  '4',
        'WAIT_SELLER_SEND_GOODS'    =>'4',// 待发出换货商品  转换成  待收货状态？没这个状态的
        'SELLER_REFUSED_CONFIRM_GOODS'=>'9',// 卖家拒绝确认收货
        'EXCHANGE_CLOSE_TO_SALES_RETURN'=>'5',//  请退款
    );



    protected function _formatAddParams($params) {
        self::trim($params);
        return $params;
    }


    protected function _returnProductAdditional($sdf) {
        return array();
    }

    public function add($params){

        $original_bn = $params['dispute_id'];
        $change_conf = app::get('ome')->getConf('shop.tmallchange.config.'.$this->__channelObj->channel['shop_id']);
        if (!$change_conf){
            $this->__apilog['result']['msg'] = '未开启接收换货业务';
            return false;

        }

        $sdf = $this->_formatAddParams($params);
        if ($sdf){
            $original_bn = $sdf['order']['order_bn'];
        }
        $this->__apilog['title'] = '店铺(' . $this->__channelObj->channel['name'] . ')换货业务处理[换货单号：' . $params['dispute_id']. ']';
        $this->__apilog['original_bn'] = $original_bn;
        $this->__apilog['result']['data'] = array('tid'=>$original_bn,'aftersale_id'=>$params['dispute_id'],'retry'=>'false');
        if(empty($sdf)) {
            $this->__apilog['result']['msg'] = '不接收售后单';
            return false;
        }
        //是否发货判断
        if (in_array($sdf['order']['ship_status'],array('0'))){
            $this->__apilog['result']['msg'] = '未发货订单不接收售后单';
            return false;
        }
        //判断状态是否处理
        $status = self::$return_status[strtoupper($sdf['status'])];
        if (!$status) {

            $this->__apilog['result']['msg'] = '此状态不处理!';
            return false;
        }
        $shopId = $sdf['shop_id'] = $this->__channelObj->channel['shop_id'];

        if(empty($sdf['return_items'])) {
            $this->__apilog['result']['msg'] = '退货明细不可为空!';
            return false;
        }

        if (is_array($sdf['change_items'])){
            $changeItemList = $this->_formatChangeItemlist($sdf['change_items']);
            if (empty($changeItemList)){
                $this->__apilog['result']['msg'] = '换货明细不可为空!';
                return false;
            }else{
                $sdf['change_items'] = $changeItemList;
            }
        }


        return $this->_returnProductAddSdf($sdf);
    }


    protected function _getOrderDelivery($sdf) {
        #获取订单关联的所有已发货的发货单delivery_id
        $sql = "SELECT dord.delivery_id, d.branch_id FROM sdb_ome_delivery_order AS dord
                  LEFT JOIN sdb_ome_delivery AS d ON(dord.delivery_id=d.delivery_id)
                  WHERE dord.order_id='".$sdf['order']['order_id']."'
                    AND (d.parent_id=0 OR d.is_bind='true')
                    AND d.disabled='false' AND d.status='succ'";

        $result = kernel::database()->select($sql);
        if(count($result) > 1 && !empty($sdf['return_items'])) {
            $arrDelivery = array();
            foreach ($result as $key => $val) {
                $arrDelivery[$val['delivery_id']] = $val;
            }
            $productId = '';
            foreach($sdf['return_items'] as $iVal) {
                if($iVal['product_id']) {
                    $productId = $iVal['product_id'];
                    break;
                }
            }
            $deliItemModel = app::get('ome')->model('delivery_items');
            $itemData = $deliItemModel->getList('delivery_id', array('delivery_id'=>array_keys($arrDelivery), 'product_id'=>$productId), 0, 1);
        }
        return $itemData ? $arrDelivery[$itemData[0]['delivery_id']] : $result[0];
    }



    protected function _returnProductAddSdf($sdf) {

        $sdf['status'] = self::$return_status[strtoupper($sdf['status'])];
        $returnModel = app::get('ome')->model('return_product');
        $tgReturn = $returnModel->getList('*', array('shop_id'=>$sdf['shop_id'],'return_bn'=>$sdf['return_bn']));
        $version_change = false;
        if($tgReturn) {
            $sdf['return_product'] = $tgReturn[0];

            $reshipData = $this->_returnProductReship($sdf['return_product']['return_id'], $sdf['order']['order_id']);
            if($reshipData['reship']) {
                $sdf['reship'] = $reshipData['reship'];
            }
            if($reshipData['other_reship_items']) {
                $sdf['other_reship_items'] = $reshipData['other_reship_items'];
            }

            if ($sdf['modified'] > $sdf['return_product']['outer_lastmodify']){
                $version_change = true;
            }

            if($sdf['modified'] <= $sdf['return_product']['outer_lastmodify']) {
                $this->__apilog['result']['msg'] = '更新时间未变化不更新';
                return false;
            }
        }
        //不存在状态为已完成和已拒绝不处理
        if (!$sdf['return_product'] && in_array($sdf['status'],array('4','5'))){
            $this->__apilog['result']['msg'] = '已完成单据不处理!';
            return false;
        }

        //可退数量判断

        $sdf['version_change'] = $version_change;
        //判断是否已拒绝或完成状态
        if ($sdf['return_product']['status'] == '5' && !$version_change){
            $this->__apilog['result']['msg'] = '已拒绝版本未变化不处理';
            return false;
        }
        if ($sdf['return_product']['status'] == '4'){
            $this->__apilog['result']['msg'] = '已完成售后单不处理';
            return false;
        }
        $delivery = $this->_getOrderDelivery($sdf);
        if ($delivery) {
            $sdf['delivery_id'] = $delivery['delivery_id'];
            $sdf['branch_id'] = $delivery['branch_id'];
        }
        if ($sdf['status'] == '1' || !$sdf['return_product']) {
            // 如果前端传了会员名
            if ($sdf['buyer_nick']) {
                $shopMemberModel = app::get('ome')->model('shop_members');
                $member = $shopMemberModel->getList('member_id', array('shop_member_id' => $sdf['buyer_nick'], 'shop_id' => $sdf['shop_id']), 0, 1);
                $sdf['member_id'] = $member[0]['member_id'];
            } else {
                $sdf['member_id'] = $sdf['order']['member_id'];
            }
            //判断数量
            $return_items = $sdf['return_items'];
            $itemObj = app::get('ome')->model('reship_items');


            foreach($return_items as $item){
                $effective = $itemObj->Get_refund_count ( $sdf['order']['order_id'], $item['bn'] );
                if ($effective<=0)  {
                    $this->__apilog['result']['msg'] = "返回值：货号[{$item['bn']}]申请数超出订单可退数";
                    return false;
                }
            }

            $sdf['table_additional'] = $this->_returnProductAdditional($sdf);
        } else {
            $returnItemModel = app::get('ome')->model('return_product_items');
            $tgReturnItems = $returnItemModel->getList('item_id',array('return_id'=>$sdf['return_product']['return_id']));
            if (!$tgReturnItems) {
                $this->__apilog['result']['msg'] = '缺少明细';
                return false;
            }

            if ($sdf['status']<=3){
                $sdf['table_additional'] = $this->_returnProductAdditional($sdf);
            }
        }
        $sdf['choose_type_flag'] = 1;

        return $sdf;
    }

    protected function _returnProductReship($returnProductId, $orderId) {
        $rs = array();
        $oReship = app::get('ome')->model('reship');
        $field = 'reship_id,reship_bn,return_id,is_check,return_logi_name,return_logi_no,outer_lastmodify,change_status,return_type,changebranch_id,branch_id';
        $reship = $oReship->getList($field, array('return_id' => $returnProductId), 0, 1);
        if($reship) {
            $rs['reship'] = $reship[0];
            $otherFilter = ' AND r.reship_id != ' . $rs['reship']['reship_id'];
        }
        $sql = "SELECT i.bn, i.num FROM sdb_ome_reship as r left join sdb_ome_reship_items as i on r.reship_id=i.reship_id WHERE i.return_type='return' AND r.is_check!='5' AND r.order_id='" . $orderId . "' " . $otherFilter;
        $otherReship = $oReship->db->select($sql);
        if($otherReship) {
            $rs['other_reship_items'] = $otherReship;
        }
        return $rs;
    }


    protected function _formatChangeItemlist($change_items){
        $salesMLib = kernel::single('material_sales_material');
        foreach($change_items as &$items){
            $product_detail = $salesMLib->getSalesMByBn($this->__channelObj->channel['shop_id'],$items['bn']);
            if($product_detail){
                $items['product_id'] = $product_detail['sm_id'];
                $items['name'] = $product_detail['sales_material_name'];
            }else{
                return array();
            }
        }
        return $change_items;
    }


}
